@@ -2,15 +2,15 @@
 import os
 from argparse import Namespace
 from collections import namedtuple
+from copy import deepcopy
 from os.path import exists
 from pathlib import Path
-from typing import Callable, Any, Union, Iterable, Dict
+from typing import Union, Iterable, Dict
 
 import yaml
+
 from ddb.config.merger import config_merger
-from deepmerge import always_merger
-from dotty_dict import dotty, Dotty
-from marshmallow import Schema
+from ddb.config.migrations import MigrationsDotty
 
 ConfigPaths = namedtuple('ConfigPaths', ['ddb_home', 'home', 'project_home'])
 
@@ -19,11 +19,12 @@ def configuration_file(path: str, filenames: Iterable[str], extensions: Iterable
     """
     Find configuration file for given path and possible filename/extensions
     """
-    for basename in filenames:
-        for ext in extensions:
-            file = os.path.join(path, basename + '.' + ext)
-            if exists(file):
-                return file
+    if path:
+        for basename in filenames:
+            for ext in extensions:
+                file = os.path.join(path, basename + '.' + ext)
+                if exists(file):
+                    return file
     return None
 
 
@@ -43,6 +44,8 @@ def get_default_config_paths(env_prefix, filenames, extensions) -> ConfigPaths:
             project_home_candidate = project_home_candidate_parent
         project_home = project_home_candidate
 
+    project_home = os.path.abspath(project_home)
+
     home = os.environ.get(env_prefix + '_HOME', os.path.join(str(Path.home()), '.docker-devbox'))
     ddb_home = os.environ.get(env_prefix + '_DDB_HOME', os.path.join(home, 'ddb'))
 
@@ -53,8 +56,9 @@ class Config:  # pylint:disable=too-many-instance-attributes
     """
     Configuration
     """
-    # Static default values for configuration, can be modified from code mainly for tests purpose
+    # Static default/overrides values for configuration, can be modified from code mainly for tests purpose
     defaults = None
+    overrides = lambda config: config
 
     def __init__(self,
                  paths: Union[ConfigPaths, None] = None,
@@ -70,12 +74,12 @@ class Config:  # pylint:disable=too-many-instance-attributes
         self.filenames = filenames
         self.extensions = extensions
         self.env_additions = {}
-        self.data = dotty()
-        self.loaded_data = {}
+        self.data = MigrationsDotty()
         self.paths = paths if paths else get_default_config_paths(env_prefix, filenames, extensions)
         self.cwd = cwd
         self.args = args if args else Namespace()
         self.unknown_args = unknown_args if unknown_args else []
+        self.env_key = 'env'
 
     def reset(self, *args, **kwargs):
         """
@@ -90,6 +94,23 @@ class Config:  # pylint:disable=too-many-instance-attributes
         self.data.clear()
         self.env_additions.clear()
 
+    @property
+    def project_configuration_file(self):
+        """
+        Retrieve the project configuration file.
+        """
+        return configuration_file(self.paths.project_home, self.filenames, self.extensions)
+
+    @property
+    def project_cwd(self):
+        """
+        Retrieve the current working directory, relative to project_home.
+        :return:
+        """
+        if self.paths.project_home and self.cwd:
+            return os.path.relpath(self.cwd, self.paths.project_home)
+        return None
+
     @property
     def eject(self):
         """
@@ -119,63 +140,63 @@ class Config:  # pylint:disable=too-many-instance-attributes
                 continue
             for basename in self.filenames:
                 for ext in self.extensions:
-                    file = os.path.join(path, basename + '.' + ext)
+                    if basename.endswith('.' + ext) and os.path.exists(os.path.join(path, basename)):
+                        file = os.path.join(path, basename)
+                    else:
+                        file = os.path.join(path, basename + '.' + ext)
                     ret.append(file)
         return ret
 
-    def read(self, env_key='env'):
+    def read(self, defaults=None, files=None):
         """
-        Read configuration data from files. Variable in 'env_key' key will be placed loaded as environment variables.
+        Read configuration data from files.
         """
-        loaded_data = {} if Config.defaults is None else dict(Config.defaults)
+        if defaults is None:
+            loaded_data = {}
+        else:
+            loaded_data = dict(defaults)
 
-        for file in self.files:
+        if files is None:
+            files = self.files
+
+        found_files = {}
+
+        for file in files:
             if exists(file):
                 with open(file, 'rb') as stream:
-                    file_data = yaml.load(stream, Loader=yaml.FullLoader)
+                    file_data = yaml.safe_load(stream)
+                    found_files[file] = self.apply_environ_overrides(deepcopy(file_data))
                     if file_data:
                         loaded_data = config_merger.merge(loaded_data, file_data)
 
-        if env_key in loaded_data:
-            env = loaded_data.pop(env_key)
-            if env:
-                for (name, value) in env.items():
-                    os.environ[name] = value
+        if Config.overrides:  # pylint:disable=using-constant-test
+            Config.overrides(loaded_data)
 
-        return self.apply_environ_overrides(loaded_data)
+        return self.apply_environ_overrides(loaded_data), found_files
 
     def load_from_data(self, data: Dict):
         """
         Load configuration data from readen files.
         """
-        self.loaded_data = dict(data)
-        self.data = dotty(always_merger.merge(self.data, data))
+        merged_data = config_merger.merge(dict(self.data.raw()), data)
+        self.data = MigrationsDotty(merged_data)
 
-    def load(self, env_key='env'):
+        if self.env_key in self.data:
+            env = self.data.pop(self.env_key)
+            if env:
+                for (name, value) in env.items():
+                    os.environ[name] = value
+
+    def load(self, defaults=None, files=None):
         """
         Load configuration data from files. Variable in 'env_key' key will be placed loaded as environment variables.
         """
-        self.load_from_data(self.read(env_key))
-
-    def sanitize_and_validate(self, schema: Schema, key: str, auto_configure: Callable[[Dotty], Any] = None):
-        """
-        Sanitize and validate using given schema part of the configuration given by configuration key.
-        """
-
-        raw_feature_config = self.data.get(key)
-
-        if not raw_feature_config:
-            raw_feature_config = {}
-        feature_config = schema.dump(raw_feature_config)
-
-        feature_config = schema.dump(feature_config)
-
-        if auto_configure:
-            auto_configure(dotty(feature_config))
-
-        feature_config = schema.load(feature_config)
-        feature_config = self.apply_environ_overrides(feature_config, self.env_override_prefix + "_" + key)
-        self.data[key] = feature_config
+        defaults_to_apply = {} if Config.defaults is None else dict(Config.defaults)
+        if defaults:
+            # Config.defaults should have the priority over given defaults.
+            defaults_to_apply = config_merger.merge(defaults, defaults_to_apply)
+        read_data, _ = self.read(defaults_to_apply, files)
+        self.load_from_data(read_data)
 
     def apply_environ_overrides(self, data, prefix=None):
         """
